@@ -1,5 +1,7 @@
 package IPC::System::Simple;
 
+# ABSTRACT: Run commands simply, with detailed diagnostics
+
 use 5.006;
 use strict;
 use warnings;
@@ -10,19 +12,40 @@ use Scalar::Util qw(tainted);
 use Config;
 use constant WINDOWS => ($^O eq 'MSWin32');
 use constant VMS     => ($^O eq 'VMS');
-use if WINDOWS, 'Win32::Process', qw(INFINITE NORMAL_PRIORITY_CLASS);
-use if WINDOWS, 'File::Spec';
-use if WINDOWS, 'Win32';
 
-# This uses the same rules as the core win32.c/get_shell() call.
+BEGIN {
 
-use if WINDOWS, 'constant', WINDOWS_SHELL => eval { Win32::IsWinNT() }
-	                                  ? [ qw(cmd.exe /x/d/c) ]
-                                          : [ qw(command.com /c) ];
+    # It would be lovely to use the 'if' module here, but it didn't
+    # enter core until 5.6.2, and we want to keep 5.6.0 compatibility.
 
-# These are used when invoking _win32_capture
-use if WINDOWS, 'constant', NO_SHELL  => 0;
-use if WINDOWS, 'constant', USE_SHELL => 1;
+
+    if (WINDOWS) {
+
+        ## no critic (ProhibitStringyEval)
+
+        eval q{
+            use Win32::Process qw(INFINITE NORMAL_PRIORITY_CLASS);
+            use File::Spec;
+            use Win32;
+            use Win32::ShellQuote;
+
+            # This uses the same rules as the core win32.c/get_shell() call.
+            use constant WINDOWS_SHELL => eval { Win32::IsWinNT() }
+                ? [ File::Spec->catfile(Win32::GetFolderPath(Win32::CSIDL_SYSTEM), 'cmd.exe'), '/x/d/c' ]
+                : [ File::Spec->catfile(Win32::GetFolderPath(Win32::CSIDL_SYSTEM), 'command.com'), '/c' ];
+
+            # These are used when invoking _win32_capture
+            use constant NO_SHELL  => 0;
+            use constant USE_SHELL => 1;
+
+        };
+
+        ## use critic
+
+        # Die nosily if any of the above broke.
+        die $@ if $@;
+    }
+}
 
 # Note that we don't use WIFSTOPPED because perl never uses
 # the WUNTRACED flag, and hence will never return early from
@@ -39,7 +62,10 @@ use constant FAIL_TAINT_ENV => q{%s called with tainted environment $ENV{%s}};
 use constant FAIL_SIGNAL    => q{"%s" died to signal "%s" (%d)%s};
 use constant FAIL_BADEXIT   => q{"%s" unexpectedly returned exit value %d};
 
-use constant FAIL_POSIX     => q{IPC::System::Simple does not understand the POSIX error '%s'.  Please check http://search.cpan.org/perldoc?IPC::System::Simple to see if there is an updated version.  If not please report this as a bug to http://rt.cpan.org/Public/Bug/Report.html?Queue=IPC-System-Simple};
+use constant FAIL_UNDEF     => q{%s called with undefined command};
+
+
+use constant FAIL_POSIX     => q{IPC::System::Simple does not understand the POSIX error '%s'.  Please check https://metacpan.org/pod/IPC::System::Simple to see if there is an updated version.  If not please report this as a bug to https://github.com/pjf/ipc-system-simple/issues};
 
 # On Perl's older than 5.8.x we can't assume that there'll be a
 # $^{TAINT} for us to check, so we assume that our args may always
@@ -61,7 +87,9 @@ our @EXPORT_OK = qw(
     $EXITVAL EXIT_ANY
 );
 
-our $VERSION = '0.16';
+our $VERSION = '1.28_001';
+$VERSION =~ tr/_//d;
+
 our $EXITVAL = -1;
 
 my @Signal_from_number = split(' ', $Config{sig_name});
@@ -75,7 +103,7 @@ if (VMS) {
 	push(@Check_tainted_env, 'DCL$PATH');
 }
 
-# Not all systems implment the WIFEXITED calls, but POSIX
+# Not all systems implement the WIFEXITED calls, but POSIX
 # will always export them (even if they're just stubs that
 # die with an error).  Test for the presence of a working
 # WIFEXITED and friends, or define our own.
@@ -83,6 +111,7 @@ if (VMS) {
 eval { WIFEXITED(0); };
 
 if ($@ =~ UNDEFINED_POSIX_RE) {
+        no warnings 'redefine';  ## no critic
 	*WIFEXITED   = sub { not $_[0] & 0xff };
 	*WEXITSTATUS = sub { $_[0] >> 8  };
 	*WIFSIGNALED = sub { $_[0] & 127 };
@@ -95,21 +124,34 @@ if ($@ =~ UNDEFINED_POSIX_RE) {
 # many systems define it.  Check the POSIX module in the hope that
 # it may actually be there.
 
+
+# TODO: Ideally, $NATIVE_WCOREDUMP should be a constant.
+
+my $NATIVE_WCOREDUMP;
+
 eval { POSIX::WCOREDUMP(1); };
 
 if ($@ =~ UNDEFINED_POSIX_RE) {
 	*WCOREDUMP = sub { $_[0] & 128 };
+        $NATIVE_WCOREDUMP = 0;
 } elsif ($@) {
 	croak sprintf FAIL_POSIX, $@;
 } else {
 	# POSIX actually has it defined!  Huzzah!
 	*WCOREDUMP = \&POSIX::WCOREDUMP;
+        $NATIVE_WCOREDUMP = 1;
+}
+
+sub _native_wcoredump {
+    return $NATIVE_WCOREDUMP;
 }
 
 # system simply calls run
 
+no warnings 'once'; ## no critic
 *system  = \&run;
 *systemx = \&runx;
+use warnings;
 
 # run is our way of running a process with system() semantics
 
@@ -126,14 +168,24 @@ sub run {
                 return systemx($valid_returns, $command, @args);
 	}
 
-        # Without arguments, we're calling system, and checking
+    if (WINDOWS) {
+        my $pid = _spawn_or_die(&WINDOWS_SHELL->[0], join ' ', @{&WINDOWS_SHELL}, $command);
+        $pid->Wait(INFINITE);	# Wait for process exit.
+        $pid->GetExitCode($EXITVAL);
+        return _check_exit($command,$EXITVAL,$valid_returns);
+    }
+
+    # Without arguments, we're calling system, and checking
         # the results.
 
 	# We're throwing our own exception on command not found, so
 	# we don't need a warning from Perl.
 
-	no warnings 'exec';		## no critic
-	CORE::system($command,@args);
+        {
+            # silence 'Statement unlikely to be reached' warning
+            no warnings 'exec';             ## no critic
+            CORE::system($command,@args);
+        }
 
 	return _process_child_error($?,$command,$valid_returns);
 }
@@ -148,7 +200,7 @@ sub runx {
     if (WINDOWS) {
         our $EXITVAL = -1;
 
-        my $pid = _spawn_or_die($command, "$command @args");
+        my $pid = _spawn_or_die($command, Win32::ShellQuote::quote_native($command, @args));
 
         $pid->Wait(INFINITE);	# Wait for process exit.
         $pid->GetExitCode($EXITVAL);
@@ -178,7 +230,7 @@ sub capture {
 
         if (WINDOWS) {
             # USE_SHELL really means "You may use the shell if you need it."
-            return _win32_capture(USE_SHELL, $valid_returns, $command, @args);
+            return _win32_capture(USE_SHELL, $valid_returns, $command);
         }
 
 	our $EXITVAL = -1;
@@ -261,12 +313,22 @@ sub _win32_capture {
         # And now we spawn our new process with inherited
         # filehandles.
 
-        my $pid = _spawn_or_die($exe, "$command @args");
+        my $err;
+        my $pid = eval { 
+                _spawn_or_die($exe, @args ? Win32::ShellQuote::quote_native($command, @args) : $command);
+        }
+        or do {
+                $err = $@;
+        };
 
-        # Now restore our STDOUT.
+        # Regardless of whether our command ran, we must restore STDOUT.
+        # RT #48319
         open(STDOUT, '>&', $saved_stdout)  ## no critic
                 or croak sprintf(FAIL_PLUMBING,"Can't restore STDOUT", $!);
 
+        # And now, if there was an actual error , propagate it.
+        die $err if defined $err;   # If there's an error from _spawn_or_die
+
         # Clean-up the filehandles we no longer need...
 
         close($write_fh)
@@ -469,6 +531,16 @@ sub _process_child_error {
 
 	my $coredump = WCOREDUMP($child_error);
 
+        # There's a bug in perl 5.10.0 where if the system
+        # does not provide a native WCOREDUMP, then $? will
+        # never contain coredump information.  This code
+        # checks to see if we have the bug, and works around
+        # it if needed.
+
+        if ($] >= 5.010 and not $NATIVE_WCOREDUMP) {
+            $coredump ||= WCOREDUMP( ${^CHILD_ERROR_NATIVE} );
+        }
+
 	if ($child_error == -1) {
 		croak sprintf(FAIL_START, $command, $!);
 
@@ -531,6 +603,10 @@ sub _process_args {
 
 	my $command = shift(@_);
 
+        if (not defined $command) {
+                croak sprintf( FAIL_UNDEF, $caller );
+        }
+
 	return ($valid_returns,$command,@_);
 
 }
@@ -581,7 +657,7 @@ you can just write:
 
     use IPC::System::Simple qw(system);
 
-and all of your C<system> commands will either succeeed (run to
+and all of your C<system> commands will either succeed (run to
 completion and return a zero exit value), or die with rich diagnostic
 messages.
 
@@ -646,12 +722,12 @@ or process diagnostics, then read on!
 
   my @lines  = capturex([0..5], "some_command", @args);
 
-=head1 ADVNACED USAGE
+=head1 ADVANCED USAGE
 
 =head2 run() and system()
 
 C<IPC::System::Simple> provides a subroutine called
-C<run>, that executes a command using the same semantics is
+C<run>, that executes a command using the same semantics as
 Perl's built-in C<system>:
 
     use IPC::System::Simple qw(run);
@@ -670,7 +746,7 @@ same behaviour:
 
     use IPC::System::Simple qw(system);
 
-    system("cat *.txt");  # system now suceeds or dies!
+    system("cat *.txt");  # system now succeeds or dies!
 
 C<system> and C<run> are aliases to each other.
 
@@ -788,7 +864,7 @@ value of the process:
 
 =head3 $EXITVAL
 
-The exit value of any command exeucted by C<IPC::System::Simple>
+The exit value of any command executed by C<IPC::System::Simple>
 can always be retrieved from the C<$IPC::System::Simple::EXITVAL>
 variable:
 
@@ -807,16 +883,30 @@ exception will also be thrown.
 
 =head2 WINDOWS-SPECIFIC NOTES
 
-As of C<IPC::System::Simple> v0.06, the C<run> subroutine I<when
-called with multiple arguments> will make available the full 32-bit
-exit value on Win32 systems.  This is different from the
-previous versions of C<IPC::System::Simple> and from Perl's
-in-build C<system()> function, which can only handle 8-bit return values.
+The C<run> subroutine make available the full 32-bit exit value on
+Win32 systems. This has been true since C<IPC::System::Simple> v0.06
+when called with multiple arguments, and since v1.25 when called with
+a single argument.  This is different from the previous versions of
+C<IPC::System::Simple> and from Perl's in-build C<system()> function,
+which can only handle 8-bit return values.
 
 The C<capture> subroutine always returns the 32-bit exit value under
 Windows.  The C<capture> subroutine also never uses the shell,
 even when passed a single argument.
 
+The C<run> subroutine always uses a shell when passed a single
+argument. On NT systems, it uses C<cmd.exe> in the system root, and on
+non-NT systems it uses C<command.com> in the system root.
+
+As of C<IPC::System::Simple> v1.25, the C<runx> and C<capturex>
+subroutines, as well as multiple-argument calls to the C<run> and
+C<capture> subroutines, have their arguments properly quoted, so that
+arugments with spaces and the like work properly. Unfortunately, this
+breaks any attempt to invoke the shell itself. If you really need to
+execute C<cmd.exe> or C<command.com>, use the single-argument form.
+For single-argument calls to C<run> and C<capture>, the argument must
+be properly shell-quoted in advance of the call.
+
 Versions of C<IPC::System::Simple> before v0.09 would not search
 the C<PATH> environment variable when the multi-argument form of
 C<run()> was called.  Versions from v0.09 onwards correctly search
@@ -850,7 +940,7 @@ The command was killed by a signal.  The name of the signal
 will be reported, or C<UNKNOWN> if it cannot be determined.  The
 signal number is always reported.  If we detected that the
 process dumped core, then the string C<and dumped core> is
-appeneded.
+appended.
 
 =item IPC::System::Simple::%s called with no arguments
 
@@ -894,6 +984,12 @@ see if an updated version of C<IPC::System::Simple> is available.
 If not, please file a bug report according to the L</Reporting bugs> section
 below.
 
+=item IPC::System::Simple::%s called with undefined command
+
+You've passed the undefined value as a command to be executed.
+While this is a very Zen-like action, it's not supported by
+Perl's current implementation.
+
 =back
 
 =head1 DEPENDENCIES
@@ -919,17 +1015,18 @@ to provide as much information as possible.  Rather than requiring
 the developer to inspect C<$?>, C<IPC::System::Simple> does the
 hard work for you.
 
-If an odd exit status is provided, you're informed of what it is.  If
-a signal kills your process, you are informed of both its name and
-number.  If tainted data or environment prevents your command from
-running, you are informed of exactly which datais 
+If an odd exit status is provided, you're informed of what it is.  If a
+signal kills your process, you are informed of both its name and number.
+If tainted data or environment prevents your command from running, you
+are informed of exactly which data or environmental variable is
+tainted.
 
 =item Exceptions on failure
 
-C<IPC::System::Simple> takes an agressive approach to error handling.
+C<IPC::System::Simple> takes an aggressive approach to error handling.
 Rather than allow commands to fail silently, exceptions are thrown
 when unexpected results are seen.  This allows for easy development
-using a try/catch style, and avoids the possibility of accidently
+using a try/catch style, and avoids the possibility of accidentally
 continuing after a failed command.
 
 =item Easy access to exit status
@@ -959,21 +1056,16 @@ if you need it, or consider using the L<autodie> module to replace
 C<system> with lexical scope.
 
 Core dumps are only checked for when a process dies due to a
-signal.  It is not believed thare exist any systems where processes
+signal.  It is not believed there are any systems where processes
 can dump core without dying to a signal.
 
 C<WIFSTOPPED> status is not checked, as perl never spawns processes
 with the C<WUNTRACED> option.
 
 Signals are not supported under Win32 systems, since they don't
-work at all like Unix signals.  Win32 singals cause commands to
+work at all like Unix signals.  Win32 signals cause commands to
 exit with a given exit value, which this modules I<does> capture.
 
-Only 8-bit values are returned when C<run()> or C<system()> 
-is called with a single value under Win32.  Multi-argument calls
-to C<run()> and C<system()>, as well as the C<runx()> and
-C<systemx()> always return the 32-bit Windows return values.
-
 =head2 Reporting bugs
 
 Before reporting a bug, please check to ensure you are using the
@@ -985,7 +1077,7 @@ L<http://rt.cpan.org/Public/Dist/Display.html?Name=IPC-System-Simple> .
 Please check to see if your bug has already been reported; if
 in doubt, report yours anyway.
 
-Submitting a patch and/or failing test case will greatly expediate
+Submitting a patch and/or failing test case will greatly expedite
 the fixing of bugs.
 
 =head1 FEEDBACK
@@ -1018,4 +1110,6 @@ This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.6.0 or,
 at your option, any later version of Perl 5 you may have available.
 
+=for Pod::Coverage WCOREDUMP
+
 =cut
